searchNodes=[{"doc":"HAP is an implementation of the HomeKit Accessory Protocol Specification. It allows for the creation of Elixir powered HomeKit accessories which can be controlled from a user's iOS device in a similar manner to commercially available HomeKit accessories such as smart light bulbs, window coverings and other smart home accessories.The HomeKit Data ModelThe data model of the HomeKit Accessory Protocol is represented as a tree structure. At the top level, a single HAP instance represents an Accessory Server. An accessory server hosts one or more Accessory Objects. Each accessory object represents a single, discrete physical accessory. In the case of directly connected devices, an accessory server typically hosts a single accessory object which represents device itself. Bridges will have one accessory object for each discrete phyisical object which they bridge to. Within HAP, an accessory server is represented by a HAP.AccessoryServer struct, and an accessory by the HAP.Accessory struct.Each accessory object contains exposes a set of Services, each of which represents a unit of functionality. As an example, a HomeKit Accessory Server which represented a ceiling fan with a light would contain one accessory object called 'Ceiling Fan', which would contain two services each representing the light and the fan. In addition to user-visible services, each accessory exposes an Accessory Information Service which contains information about the service's name, manufacturer, serial number and other properties. Within HAP, a service is represented by a HAP.Service struct.A service is made up of one or more Characteristics, each of which represnts a specific aspect of the given service. For example, a light bulb service exposes an On Characteristic, which is a boolean value reflecting the current on or off state of the light. If it is a dimmable light, it may also expose a Brightness Characteristic. If it is a color changing light, it may also expose a Hue Characteristic. Within HAP, a characteristic is represented by a HAP.Characteristic struct.Using HAPHAP provides a high-level interface to the HomeKit Accessory Protocol, allowing an application to present any number of services &amp; characteristics of those services to an iOS HomeKit controller. HAP is intended to be embedded within a host application which is responsible for providing the actual backing implementations of the various characteristics exposed via HomeKit. These are provided to HAP in the form of HAP.ValueStore implementations. For example, consider a Nerves application which exposes itself to HomeKit as a light bulb. Assume that the actual physical control of the light is controlled by GPIO pin 23. A typical configuration of HAP would look something like this:hap_server_config = HAP.build_accessory_server( name: &quot;My HAP Demo Device&quot;, model: &quot;HAP Demo Device&quot;, identifier: &quot;11:22:33:44:12:66&quot;, accessory_type: 5, accessories: [ HAP.build_accessory( name: &quot;HAP Lightbulb&quot;, model: &quot;HAP Lightbulb&quot;, manufacturer: &quot;HAP Inc.&quot;, serial_number: &quot;123456&quot;, firmware_revision: &quot;1.0&quot;, services: [ HAP.Services.LightBulb.build_service(MyApp.Lightbulb, gpio_pin: 23) ] ) ] ) children = [ {HAP, hap_server_config} ] Supervisor.start_link(children, opts) ...In this example, the application developer is responsible for creating a MyApp.Lightbulb module which implements the HAP.ValueStore behaviour. This module would be called by HAP whenever it needs to change or query the current state of the light. The extra options (gpio_pin: 23 in the above example) are conveyed to this module on every call, allowing a single value store implementation to service any number of characteristics or services.HAP provides convenience functions to instantiate the most common services, such as light bulbs, switches, and other common accessories &amp; services. Behind the scenes, these convenience functions return simple HAP.Accessory, HAP.Service, and HAP.Characteristic structs. This flexibility makes it possible to define the most common use cases via easy to use function calls, while also allowing more in-depth use of hand-crafted structures if HAP doesn't provide convenience functions out of the box. For more information, consult the functions defined on the HAP module, as well as the type definitions for t:HAP.AccessoryServer, t:HAP.Accessory, t:HAP.Service, and t:HAP.Characteristic.","ref":"HAP.html","title":"HAP","type":"module"},{"doc":"Builds an instance of HAP.Accessory based on the provided information. The services passed in services will be included in the created accessory's list of services, as well as an instance of Services.AccessoryInformation and Services.ProtocolInformation based on the extra provided metadata. This function is typically called within a call to HAP.build_accessory_server/1 as in the example at the top of this file.Supported fields include:name: The name to assign to this accessory, for example 'Ceiling Fan'model: The model name to assign to this accessory, for example 'FanCo Whisper III'manufacturer: The manufacturer of this accessory, for example 'FanCo'serial_number: The serial number of this accessory, for example '0012345'firmware_revision: The firmware revision of this accessory, for example '1.0'services: A list of services to include in this accessory","ref":"HAP.html#build_accessory/1","title":"HAP.build_accessory/1","type":"function"},{"doc":"Builds an instance of HAP.AccessoryServer based on the provided information. This function is typically called to build the input to a HAP.start_link/1 call as in the example at the top of this file.Supported fields include:name: The name to assign to this device, for example 'HAP Bridge'model: The model name to assign to this accessory, for example 'HAP Bridge'identifier: A unique identifier string in the form &quot;AA:BB:CC:DD:EE:FF&quot;pairing_code: A pairing code of the form 123-45-678 to be used for pairing. If not specified one will be defined dynamically.setup_id: A 4 character string used as part of the accessory discovery process. If not specified one will be defined dynamically.display_module: An optional implementation of HAP.Display used to present pairing and other information to the user. If not specified then a basic console-based display is used.data_path: The path to where HAP will store its internal data. Will be created if it does not exist. If not specified, hap_data is used.accessory_type: A HAP specified value indicating the primary function of this device. See HAP.AccessoryServer.accessory_type/0 for detailsaccessories: A list of accessories to include in this accessory server","ref":"HAP.html#build_accessory_server/1","title":"HAP.build_accessory_server/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"HAP.html#child_spec/1","title":"HAP.child_spec/1","type":"function"},{"doc":"Callback implementation for Supervisor.init/1.","ref":"HAP.html#init/1","title":"HAP.init/1","type":"function"},{"doc":"Starts a HAP instance based on the passed config","ref":"HAP.html#start_link/1","title":"HAP.start_link/1","type":"function"},{"doc":"Represents a single accessory object, containing a number of services","ref":"HAP.Accessory.html","title":"HAP.Accessory","type":"module"},{"doc":"Represents an accessory consisting of a number of services","ref":"HAP.Accessory.html#t:t/0","title":"HAP.Accessory.t/0","type":"type"},{"doc":"Represents a top-level HAP instance configuration","ref":"HAP.AccessoryServer.html","title":"HAP.AccessoryServer","type":"module"},{"doc":"A unique identifier string in the form &quot;AA:BB:CC:DD:EE:FF&quot;","ref":"HAP.AccessoryServer.html#t:accessory_identifier/0","title":"HAP.AccessoryServer.accessory_identifier/0","type":"type"},{"doc":"A HAP specified value indicating the primary function of this device as found in Section 13 of Apple's HomeKit Accessory Protocol Specification. Valid values include:OtherBridgeFanGarageLightbulbDoor LockOutletSwitchThermostatSensorSecurity SystemDoorWindowWindow CoveringProgrammable SwitchRange ExtenderIP CameraVideo Door BellAir PurifierHeaterAir ConditionerHumidifierDehumidifierSprinklerFaucetShower SystemRemote","ref":"HAP.AccessoryServer.html#t:accessory_type/0","title":"HAP.AccessoryServer.accessory_type/0","type":"type"},{"doc":"The model of an accessory server","ref":"HAP.AccessoryServer.html#t:model/0","title":"HAP.AccessoryServer.model/0","type":"type"},{"doc":"The name of an accessory server","ref":"HAP.AccessoryServer.html#t:name/0","title":"HAP.AccessoryServer.name/0","type":"type"},{"doc":"A pairing code of the form 123-45-678","ref":"HAP.AccessoryServer.html#t:pairing_code/0","title":"HAP.AccessoryServer.pairing_code/0","type":"type"},{"doc":"A pairing URL suitable for display in a QR code","ref":"HAP.AccessoryServer.html#t:pairing_url/0","title":"HAP.AccessoryServer.pairing_url/0","type":"type"},{"doc":"Represents an accessory server consisting of a number of accessories","ref":"HAP.AccessoryServer.html#t:t/0","title":"HAP.AccessoryServer.t/0","type":"type"},{"doc":"A behaviour which encapsulates all user-facing display concerns for an accessory. Applications which use HAP may provide their own implementation of this behaviour as an argument to HAP.build_accessory_server/1. If no such implementation is provided HAP uses a default console based implementation found at HAP.ConsoleDisplay","ref":"HAP.Display.html","title":"HAP.Display","type":"behaviour"},{"doc":"Stop displaying any currently displayed pairing information to the user. This is most commonly because a pairing has been established with a controller","ref":"HAP.Display.html#c:clear_pairing_code/0","title":"HAP.Display.clear_pairing_code/0","type":"callback"},{"doc":"Display a notification to the user containing information on how to pair with this accessory server. The QR code often seen on HomeKit devices to facilitate pairing contains the value of pairing_url; see HAP.ConsoleDisplay for an example of how to display it to the user.","ref":"HAP.Display.html#c:display_pairing_code/3","title":"HAP.Display.display_pairing_code/3","type":"callback"},{"doc":"Display a notification to the user that identifies the named device or accessory. This comes from a user request within the Home app to identify the given device or accessory.","ref":"HAP.Display.html#c:identify/1","title":"HAP.Display.identify/1","type":"callback"},{"doc":"Defines the behaviour required of a module that wishes to act as the backing data store for a given HomeKit characteristic","ref":"HAP.ValueStore.html","title":"HAP.ValueStore","type":"behaviour"},{"doc":"Return the value of a value hosted by this value store. The passed list of opts is as specified in the hosting HAP.Configuration and can be used to distinguish a particular value within a larger value store (perhaps by GPIO pin or similar)Returns the value stored by this value store","ref":"HAP.ValueStore.html#c:get_value/1","title":"HAP.ValueStore.get_value/1","type":"callback"},{"doc":"Sets the value of a value hosted by this value store. The passed list of opts is as specified in the hosting HAP.Configuration and can be used to distinguish a particular value within a larger value store (perhaps by GPIO pin or similar)Returns :ok or {:error, reason}","ref":"HAP.ValueStore.html#c:put_value/2","title":"HAP.ValueStore.put_value/2","type":"callback"},{"doc":"","ref":"HAP.ValueStore.html#t:opts/0","title":"HAP.ValueStore.opts/0","type":"type"},{"doc":"","ref":"HAP.ValueStore.html#t:t/0","title":"HAP.ValueStore.t/0","type":"type"},{"doc":"Represents a single service, containing a number of characteristics","ref":"HAP.Service.html","title":"HAP.Service","type":"module"},{"doc":"Represents a service of a given type, containing a number of characteristics","ref":"HAP.Service.html#t:t/0","title":"HAP.Service.t/0","type":"type"},{"doc":"The type of a service as defined in Section 6.6.1 of Apple's HomeKit Accessory Protocol Specification.","ref":"HAP.Service.html#t:type/0","title":"HAP.Service.type/0","type":"type"},{"doc":"Factory for the public.hap.service.accessory-information service","ref":"HAP.Services.AccessoryInformation.html","title":"HAP.Services.AccessoryInformation","type":"module"},{"doc":"","ref":"HAP.Services.AccessoryInformation.html#build_service/1","title":"HAP.Services.AccessoryInformation.build_service/1","type":"function"},{"doc":"Factory for the public.hap.service.lightbulb service","ref":"HAP.Services.LightBulb.html","title":"HAP.Services.LightBulb","type":"module"},{"doc":"","ref":"HAP.Services.LightBulb.html#build_service/2","title":"HAP.Services.LightBulb.build_service/2","type":"function"},{"doc":"Factory for the public.hap.service.protocol.information.service service","ref":"HAP.Services.ProtocolInformation.html","title":"HAP.Services.ProtocolInformation","type":"module"},{"doc":"","ref":"HAP.Services.ProtocolInformation.html#build_service/1","title":"HAP.Services.ProtocolInformation.build_service/1","type":"function"},{"doc":"Represents a single characteristic optionally backed by an instance of ValueStore","ref":"HAP.Characteristic.html","title":"HAP.Characteristic","type":"module"},{"doc":"The format of a characteristic as defined in Table 6.5 of Apple's HomeKit Accessory Protocol Specification. One of bool, uint8, uint16, uint32, uint64, int, float, string, tlv8, or data","ref":"HAP.Characteristic.html#t:format/0","title":"HAP.Characteristic.format/0","type":"type"},{"doc":"A permission of a characteristic as defined in Table 6.4 of Apple's HomeKit Accessory Protocol Specification. One of pr, pw, ev, aa, tw, hd, or wr","ref":"HAP.Characteristic.html#t:perm/0","title":"HAP.Characteristic.perm/0","type":"type"},{"doc":"Represents a single characteristic optionally backed by an instance of ValueStore","ref":"HAP.Characteristic.html#t:t/0","title":"HAP.Characteristic.t/0","type":"type"},{"doc":"The type of a characteristic as defined in Section 6.6.1 of Apple's HomeKit Accessory Protocol Specification.","ref":"HAP.Characteristic.html#t:type/0","title":"HAP.Characteristic.type/0","type":"type"},{"doc":"The value of a characrteristic","ref":"HAP.Characteristic.html#t:value/0","title":"HAP.Characteristic.value/0","type":"type"},{"doc":"Factory for the public.hap.characteristic.firmware.revision characteristic","ref":"HAP.Characteristics.FirmwareRevision.html","title":"HAP.Characteristics.FirmwareRevision","type":"module"},{"doc":"","ref":"HAP.Characteristics.FirmwareRevision.html#build_characteristic/1","title":"HAP.Characteristics.FirmwareRevision.build_characteristic/1","type":"function"},{"doc":"Factory for the public.hap.characteristic.identify characteristic","ref":"HAP.Characteristics.Identify.html","title":"HAP.Characteristics.Identify","type":"module"},{"doc":"","ref":"HAP.Characteristics.Identify.html#build_characteristic/2","title":"HAP.Characteristics.Identify.build_characteristic/2","type":"function"},{"doc":"Factory for the public.hap.characteristic.manufacturer characteristic","ref":"HAP.Characteristics.Manufacturer.html","title":"HAP.Characteristics.Manufacturer","type":"module"},{"doc":"","ref":"HAP.Characteristics.Manufacturer.html#build_characteristic/1","title":"HAP.Characteristics.Manufacturer.build_characteristic/1","type":"function"},{"doc":"Factory for the public.hap.characteristic.model characteristic","ref":"HAP.Characteristics.Model.html","title":"HAP.Characteristics.Model","type":"module"},{"doc":"","ref":"HAP.Characteristics.Model.html#build_characteristic/1","title":"HAP.Characteristics.Model.build_characteristic/1","type":"function"},{"doc":"Factory for the public.hap.characteristic.name characteristic","ref":"HAP.Characteristics.Name.html","title":"HAP.Characteristics.Name","type":"module"},{"doc":"","ref":"HAP.Characteristics.Name.html#build_characteristic/1","title":"HAP.Characteristics.Name.build_characteristic/1","type":"function"},{"doc":"Factory for the public.hap.characteristic.on characteristic","ref":"HAP.Characteristics.On.html","title":"HAP.Characteristics.On","type":"module"},{"doc":"","ref":"HAP.Characteristics.On.html#build_characteristic/2","title":"HAP.Characteristics.On.build_characteristic/2","type":"function"},{"doc":"Factory for the public.hap.characteristic.serial-number characteristic","ref":"HAP.Characteristics.SerialNumber.html","title":"HAP.Characteristics.SerialNumber","type":"module"},{"doc":"","ref":"HAP.Characteristics.SerialNumber.html#build_characteristic/1","title":"HAP.Characteristics.SerialNumber.build_characteristic/1","type":"function"},{"doc":"Factory for the public.hap.characteristic.version characteristic","ref":"HAP.Characteristics.Version.html","title":"HAP.Characteristics.Version","type":"module"},{"doc":"","ref":"HAP.Characteristics.Version.html#build_characteristic/1","title":"HAP.Characteristics.Version.build_characteristic/1","type":"function"}]